/*
 * JBoss, Home of Professional Open Source
 * Copyright 2012, Red Hat, Inc. and/or its affiliates, and individual
 * contributors by the @authors tag. See the copyright.txt in the 
 * distribution for a full listing of individual contributors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jboss.as.quickstarts.mdb;

import java.util.logging.Logger;

import javax.annotation.Resource;
import javax.ejb.ActivationConfigProperty;
import javax.ejb.MessageDriven;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.jms.Topic;

import org.jboss.ejb3.annotation.ResourceAdapter;

/**
 * <p>
 * A simple Message Driven Bean that asynchronously receives and processes the
 * messages that are sent to the queue.
 * </p>
 * 
 * @author Serge Pagop (spagop@redhat.com)
 * 
 */
@ResourceAdapter("ConnectionFactory1Mgmt")
@MessageDriven(name = "HelloWorldQueueMDB", activationConfig = {
		@ActivationConfigProperty(propertyName = "reconnectAttempts", propertyValue = "-1"),
		@ActivationConfigProperty(propertyName = "hA", propertyValue = "true"),
		@ActivationConfigProperty(propertyName = "destinationType", propertyValue = "javax.jms.Queue"),
		@ActivationConfigProperty(propertyName = "destination", propertyValue = "/queue/exampleQueue"),
		@ActivationConfigProperty(propertyName = "acknowledgeMode", propertyValue = "Auto-acknowledge") })
public class HelloWorldQueueMDB implements MessageListener {

	private final static Logger LOGGER = Logger
			.getLogger(HelloWorldQueueMDB.class.toString());

	@Resource(mappedName = "java:/ConnectionFactory1Mgmt")
	private ConnectionFactory connectionFactory;

	Topic replyTopic;

	/**
	 * @see MessageListener#onMessage(Message)
	 */
	@TransactionAttribute(TransactionAttributeType.REQUIRED)
	public void onMessage(Message rcvMessage) {
		TextMessage msg = null;
		try {
			if (rcvMessage instanceof TextMessage) {
				msg = (TextMessage) rcvMessage;

				LOGGER.info("Received Message from queue: " + msg.getText());
			} else {
				LOGGER.warning("Message of wrong type: "
						+ rcvMessage.getClass().getName());
			}
		} catch (JMSException e) {
			throw new RuntimeException(e);
		}

		if (msg != null) {
			Connection connection = null;
			Session session = null;
			MessageProducer messageProducer = null;
			try {
				String text = msg.getText();

				Destination destination;

				// InitialContext ic = new InitialContext();
				// ConnectionFactory connectionFactory =
				// (ConnectionFactory)ic.lookup("java:/ConnectionFactory1Mgmt");

				connection = connectionFactory.createConnection();
				session = connection.createSession(false,
						Session.AUTO_ACKNOWLEDGE);
				destination = session.createTopic("exampleTopic");

				messageProducer = session.createProducer(destination);
				connection.start();
				TextMessage message = session.createTextMessage();

				message.setText(text);
				messageProducer.send(message);

				LOGGER.warning("Message PUBLISHED: " + text);

			} catch (Exception e) {
				throw new RuntimeException(e);
			} finally {
//				if (messageProducer != null) {
//					try {
//						messageProducer.close();
//					} catch (JMSException e) {
//						throw new RuntimeException(e);
//					}
//				}
//
//				if (session != null) {
//					try {
//						session.close();
//					} catch (JMSException e) {
//						throw new RuntimeException(e);
//					}
//				}

				if (connection != null) {
					try {
						connection.close();
					} catch (JMSException e) {
						throw new RuntimeException(e);
					}
				}
			}
		}
	}
}
